from django import forms
from django.utils.translation import gettext_lazy as _

from servala.core.models import (
    CloudProvider,
    ControlPlane,
    Service,
    ServiceCategory,
    ServiceOffering,
)


class ServiceFilterForm(forms.Form):
    category = forms.ModelChoiceField(
        queryset=ServiceCategory.objects.all(), required=False
    )
    cloud_provider = forms.ModelChoiceField(
        queryset=CloudProvider.objects.all(), required=False
    )
    q = forms.CharField(required=False)

    def filter_queryset(self, queryset):
        if category := self.cleaned_data.get("category"):
            queryset = queryset.filter(category=category)
        if cloud_provider := self.cleaned_data.get("cloud_provider"):
            queryset = queryset.filter(offerings__provider=cloud_provider)
        return queryset


class ControlPlaneSelectForm(forms.Form):
    control_plane = forms.ModelChoiceField(
        queryset=ControlPlane.objects.none(),
        label=_("Service Provider Zone"),
        empty_label=None,
    )

    def __init__(self, *args, planes=None, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["control_plane"].queryset = planes


class ServiceInstanceFilterForm(forms.Form):
    name = forms.CharField(required=False, label=_("Name"))
    service = forms.ModelChoiceField(
        queryset=Service.objects.all(), required=False, label=_("Service")
    )
    provider = forms.ModelChoiceField(
        queryset=ServiceOffering.objects.all()
        .values_list("provider", flat=True)
        .distinct(),
        required=False,
        label=_("Provider"),
    )
    control_plane = forms.ModelChoiceField(
        queryset=ControlPlane.objects.all(),
        required=False,
        label=_("Service Provider Zone"),
    )
    status = forms.ChoiceField(
        choices=(
            ("active", _("Active")),
            ("deleted", _("Deleted")),
        ),
        required=False,
        label=_("Status"),
    )

    def filter_queryset(self, queryset):
        if self.is_valid():
            data = self.cleaned_data
            if data["name"]:
                queryset = queryset.filter(name__icontains=data["name"])
            if data["service"]:
                queryset = queryset.filter(
                    context__service_definition__service=data["service"]
                )
            if data["provider"]:
                queryset = queryset.filter(
                    context__service_offering__provider=data["provider"]
                )
            if data["control_plane"]:
                queryset = queryset.filter(context__control_plane=data["control_plane"])
            if data["status"]:
                if data["status"] == "active":
                    queryset = queryset.filter(is_deleted=False)
                else:
                    queryset = queryset.filter(is_deleted=True)
        return queryset


class ServiceInstanceDeleteForm(forms.Form):
    name_confirm = forms.CharField(
        label=_("Instance Name"),
        max_length=63,
        widget=forms.TextInput(attrs={"class": "form-control"}),
    )

    def __init__(self, *args, **kwargs):
        self.instance_name = kwargs.pop("instance_name", None)
        super().__init__(*args, **kwargs)
        if self.instance_name:
            self.fields["name_confirm"].help_text = _(
                "To confirm deletion, please type the instance name: <strong>{instance_name}</strong>"
            ).format(instance_name=self.instance_name)
        else:
            self.fields["name_confirm"].help_text = _(
                "Please type the instance name to confirm deletion."
            )

    def clean_name_confirm(self):
        entered_name = self.cleaned_data.get("name_confirm")
        if not self.instance_name:
            raise ValidationError(
                _("Cannot confirm deletion: instance name not provided to form.")
            )
        if entered_name != self.instance_name:
            raise ValidationError(
                _("The entered name does not match the instance name. Deletion not confirmed.")
            )
        return entered_name
